from __future__ import print_function
import os
import sys
from utility import pypyodbc
from qgis.core import *
from qgis.gui import *
from datetime import datetime

class Syncronizer(object):

	def __init__(self):
		super(Syncronizer, self).__init__()

		self._connection = None

	def run(self):
		if u'Microsoft Access Driver (*.mdb)' in pypyodbc.drivers():
			driver = u'Microsoft Access Driver (*.mdb)'
		if u'Microsoft Access Driver (*.mdb, *.accdb)' in pypyodbc.drivers():
			driver = u'Microsoft Access Driver (*.mdb, *.accdb)'

		mdb = os.path.join(os.path.split(os.getcwd())[0], "Indagini", "CdI_Tabelle.mdb")
		backupDir = os.path.join(os.path.split(os.getcwd())[0], "Indagini","backup_cdi_tabelle")
		if not os.path.exists(backupDir):
			os.makedirs(backupDir)
		t = datetime.today()
		today = t.strftime('%Y%m%d%H%M') + "_"
		old_cdi = today+"Cdi_tabelle.mdb"
		os.rename(mdb,os.path.join(backupDir,old_cdi))
		pypyodbc.win_compact_mdb(os.path.join(backupDir,old_cdi),mdb,"General\0","Chaira")
		
		passwd = "Chaira"
		connStr = "DRIVER=%s;DBQ=%s;PWD=%s" % (driver, mdb, passwd)

		self._connection = pypyodbc.connect(connStr)

		# Recupero i layers da sincronizzare dalla toc
		for key, value in QgsMapLayerRegistry.instance().mapLayers().items():
			if key.lower().startswith('ind_pu'):
				indPu = value
			elif key.lower().startswith('ind_ln'):
				indLn = value
			else:
				continue

		puLayerData = self.getLayerData(indPu)
		lnLayerData = self.getLayerData(indLn)

		# Recupero i dati attualmente contenuti nel db
		puMdbData = self.getMdbData(indPu)
		lnMdbData = self.getMdbData(indLn)

		puDataSync = self.calculateDataSync(puLayerData, puMdbData)
		lnDataSync = self.calculateDataSync(lnLayerData, lnMdbData)

		result = {}
		try:
			result["Sito_Puntuale"] = self.syncData(puDataSync, 'Sito_Puntuale')
			result["Sito_Lineare"] = self.syncData(lnDataSync, 'Sito_Lineare')
			return result
		finally:
			self._connection.close()

	def getLayerData(self, vectorLayer):
		name = vectorLayer.name()
		idField = 'ID_SPU' if name.lower() == 'ind_pu' else 'ID_SLN'
		dataList = []
		exp = QgsExpression('"{}" IS NOT NULL'.format(idField))
		request = QgsFeatureRequest(exp)
		features = vectorLayer.getFeatures(request)
		for feature in features:
			geom = feature.geometry()
			if geom.type() == QGis.Point:
				xCoord, yCoord = geom.asPoint()
				singleData = (str(feature['ID_SPU']), xCoord, yCoord)
			elif geom.type() == QGis.Line:
				vertex = geom.asPolyline()
				xStart, yStart = vertex[0]
				xEnd, yEnd = vertex[-1]
				singleData = (str(feature['ID_SLN']), xStart, yStart, xEnd, yEnd)
			dataList.append(singleData)
		return dataList

	def getMdbData(self, vectorLayer):
		mdbData = []
		name = vectorLayer.name()
		idField = 'ID_SPU' if name.lower() == 'ind_pu' else 'ID_SLN'
		tableName = 'Sito_Puntuale' if name.lower() == 'ind_pu' else 'Sito_Lineare'
		sql = "SELECT %s FROM %s" % (idField, tableName)
		cursor = self._connection.cursor()
		result = cursor.execute(sql)
		for row in result:
			mdbData.append(str(row[0]))
		return mdbData

	def calculateDataSync(self, layerData, mdbData):
		toInsert = []
		toUpdate = []
		toDelete = []
		layer = layerData[:]
		mdb = mdbData[:]
		for single in layer:
			if single[0] in mdb:
				toUpdate.append(single)
				remove = mdb.remove(single[0])
			else:
				toInsert.append(single)
		toDelete = mdb
		print("Record da inserire: %d" % len(toInsert))
		print("Record da aggiornare: %d" % len(toUpdate))
		print("Record da eliminare: %d" % len(toDelete))
		return toInsert, toUpdate, toDelete

	def syncData(self, dataToSync, table):
		resultDict = {"deleted": 0, "inserted": 0, "updated": 0}
		print("Inizio sincronizzazione per la tabella %s" % table)
		insertData = dataToSync[0]
		updateData = dataToSync[1]
		deleteData = dataToSync[2]
		cursor = self._connection.cursor()
		if table.lower() == 'sito_lineare':
			sqlInsert = """INSERT INTO Sito_Lineare
							(pkey_sln, ubicazione_prov, ubicazione_com, ID_SLN,
								Acoord_X, Acoord_Y, Bcoord_X, Bcoord_Y) VALUES
								(%d, '%s', '%s', '%s', %f, %f, %f, %f)"""
			sqlUpdate = """UPDATE Sito_Lineare SET
							ubicazione_prov = '%s',
							ubicazione_com = '%s',
							Acoord_X = %f,
							Acoord_Y = %f,
							Bcoord_X = %f,
							Bcoord_Y = %f
							WHERE ID_SLN = '%s'"""
			sequenceDelete = ["""DELETE FROM Parametri_Lineari where pkey_parln in (SELECT Parametri_Lineari.pkey_parln
									FROM Sito_Lineare LEFT JOIN (Indagini_Lineari LEFT JOIN Parametri_Lineari ON Indagini_Lineari.pkey_indln = Parametri_Lineari.pkey_indln)
									ON Sito_Lineare.pkey_sln = Indagini_Lineari.pkey_sln where ID_SLN = '%s');""",
								"""DELETE FROM Indagini_Lineari where pkey_indln in (SELECT Indagini_Lineari.pkey_indln
									FROM Sito_Lineare LEFT JOIN (Indagini_Lineari LEFT JOIN Parametri_Lineari ON Indagini_Lineari.pkey_indln = Parametri_Lineari.pkey_indln)
									ON Sito_Lineare.pkey_sln = Indagini_Lineari.pkey_sln where ID_SLN = '%s');"""]
			sqlDelete = """DELETE FROM Sito_Lineare WHERE ID_SLN = '%s'"""
		elif table.lower() == 'sito_puntuale':
			sqlInsert = """INSERT INTO Sito_Puntuale
							(pkey_spu, ubicazione_prov, ubicazione_com, ID_SPU,
								coord_X, coord_Y) VALUES
								(%d, '%s', '%s', '%s', %f, %f)"""
			sqlUpdate = """UPDATE Sito_Puntuale SET
							ubicazione_prov = '%s',
							ubicazione_com = '%s',
							coord_X = %f,
							coord_Y = %f
							WHERE ID_SPU = '%s'"""
			sequenceDelete =  ["""DELETE FROM Curve where pkey_curve in (SELECT Curve.pkey_curve
						FROM Sito_Puntuale LEFT JOIN ((Indagini_Puntuali LEFT JOIN Parametri_Puntuali ON Indagini_Puntuali.pkey_indpu = Parametri_Puntuali.pkey_indpu)
						LEFT JOIN Curve ON Parametri_Puntuali.pkey_parpu = Curve.pkey_parpu) ON Sito_Puntuale.pkey_spu = Indagini_Puntuali.pkey_spu where ID_SPU = '%s');""",
					"""DELETE FROM Parametri_Puntuali where pkey_parpu in (SELECT Parametri_Puntuali.pkey_parpu
						FROM Sito_Puntuale LEFT JOIN ((Indagini_Puntuali LEFT JOIN Parametri_Puntuali ON Indagini_Puntuali.pkey_indpu = Parametri_Puntuali.pkey_indpu)
						LEFT JOIN Curve ON Parametri_Puntuali.pkey_parpu = Curve.pkey_parpu) ON Sito_Puntuale.pkey_spu = Indagini_Puntuali.pkey_spu where ID_SPU = '%s');""",
					"""DELETE FROM Indagini_Puntuali where pkey_indpu in (SELECT Indagini_Puntuali.pkey_indpu
						FROM Sito_Puntuale LEFT JOIN ((Indagini_Puntuali LEFT JOIN Parametri_Puntuali ON Indagini_Puntuali.pkey_indpu = Parametri_Puntuali.pkey_indpu)
						LEFT JOIN Curve ON Parametri_Puntuali.pkey_parpu = Curve.pkey_parpu) ON Sito_Puntuale.pkey_spu = Indagini_Puntuali.pkey_spu where ID_SPU = '%s');"""]
			sqlDelete = """DELETE FROM Sito_Puntuale WHERE ID_SPU = '%s'"""

		if len(deleteData) == 0:
			print("Nessun record da eliminare")
		else:
			for id_s in deleteData:
				for singleDelete in sequenceDelete:
					result = cursor.execute(singleDelete % id_s)
				result = cursor.execute(sqlDelete % id_s)
			print("Eliminati %d record" % len(deleteData))
			resultDict["deleted"] = len(deleteData)

		cursor.commit()
		self._connection.commit()

		if len(insertData) == 0:
			print("Nessun nuovo record da inserire")
		else:
			count = 0
			for row in insertData:
				ID_S = row[0]
				prov = ID_S[:3]
				com = ID_S[3:6]
				pkey = int(ID_S[7:])
				Acoord_X = float(row[1])
				Acoord_Y = float(row[2])
				if table.lower() == 'sito_lineare':
					Bcoord_X = float(row[3])
					Bcoord_Y = float(row[4])
					result = cursor.execute(sqlInsert % (pkey, prov, com, ID_S,
									Acoord_X, Acoord_Y, Bcoord_X, Bcoord_Y))
					count += 1
					continue
				result = cursor.execute(sqlInsert % (pkey, prov, com, ID_S,
									Acoord_X, Acoord_Y))
				count += 1
			print("Inseriti %d record" % count)
			resultDict["inserted"] = count

		cursor.commit()
		self._connection.commit()

		if len(updateData) == 0:
			print("Nessun record da aggiornare")
		else:
			count = 0
			for row in updateData:
				ID_S = row[0]
				prov = ID_S[:3]
				com = ID_S[3:6]
				pkey = int(ID_S[7:])
				Acoord_X = float(row[1])
				Acoord_Y = float(row[2])
				if table.lower() == 'sito_lineare':
					Bcoord_X = float(row[3])
					Bcoord_Y = float(row[4])
					result = cursor.execute(sqlUpdate % (prov, com,
									Acoord_X, Acoord_Y, Bcoord_X, Bcoord_Y, ID_S))
					count += 1
					continue
				result = cursor.execute(sqlUpdate % (prov, com,
									Acoord_X, Acoord_Y, ID_S))
				count += 1
			print("Aggiornati %d record" % count)
			resultDict["updated"] = count

		cursor.commit()
		self._connection.commit()

		print("Sincronizzazione per la tabella %s completata" % table)
		return resultDict
